@@ -768,6 +768,22 @@ class TypeParserTest extends TestCase
 					new IdentifierTypeNode('Foo')
 				),
 			],
+			[
+				'pure-callable(): Foo',
+				new CallableTypeNode(
+					new IdentifierTypeNode('pure-callable'),
+					[],
+					new IdentifierTypeNode('Foo')
+				),
+			],
+			[
+				'pure-Closure(): Foo',
+				new CallableTypeNode(
+					new IdentifierTypeNode('pure-Closure'),
+					[],
+					new IdentifierTypeNode('Foo')
+				),
+			],
 			[
 				'callable(): ?Foo',
 				new CallableTypeNode(
@@ -1051,6 +1067,13 @@ class TypeParserTest extends TestCase
 					new IdentifierTypeNode('int')
 				),
 			],
+			[
+				'self::TYPES[ int ]',
+				new OffsetAccessTypeNode(
+					new ConstTypeNode(new ConstFetchNode('self', 'TYPES')),
+					new IdentifierTypeNode('int')
+				),
+			],
 			[
 				"?\t\xA009", // edge-case with \h
 				new NullableTypeNode(
