@@ -17,6 +17,9 @@ use PHPStan\PhpDocParser\Ast\PhpDoc\Doctrine\DoctrineArgument;
 use PHPStan\PhpDocParser\Ast\PhpDoc\Doctrine\DoctrineArray;
 use PHPStan\PhpDocParser\Ast\PhpDoc\Doctrine\DoctrineArrayItem;
 use PHPStan\PhpDocParser\Ast\PhpDoc\MethodTagValueNode;
+use PHPStan\PhpDocParser\Ast\PhpDoc\ParamClosureThisTagValueNode;
+use PHPStan\PhpDocParser\Ast\PhpDoc\ParamImmediatelyInvokedCallableTagValueNode;
+use PHPStan\PhpDocParser\Ast\PhpDoc\ParamLaterInvokedCallableTagValueNode;
 use PHPStan\PhpDocParser\Ast\PhpDoc\ParamTagValueNode;
 use PHPStan\PhpDocParser\Ast\PhpDoc\PhpDocNode;
 use PHPStan\PhpDocParser\Ast\PhpDoc\PhpDocTagNode;
@@ -1665,6 +1668,78 @@ class PrinterTest extends TestCase
 
 			},
 		];
+
+		yield [
+			'/** @param-immediately-invoked-callable $foo test */',
+			'/** @param-immediately-invoked-callable $bar foo */',
+			new class extends AbstractNodeVisitor {
+
+				public function enterNode(Node $node)
+				{
+					if ($node instanceof ParamImmediatelyInvokedCallableTagValueNode) {
+						$node->parameterName = '$bar';
+						$node->description = 'foo';
+					}
+
+					return $node;
+				}
+
+			},
+		];
+
+		yield [
+			'/** @param-later-invoked-callable $foo test */',
+			'/** @param-later-invoked-callable $bar foo */',
+			new class extends AbstractNodeVisitor {
+
+				public function enterNode(Node $node)
+				{
+					if ($node instanceof ParamLaterInvokedCallableTagValueNode) {
+						$node->parameterName = '$bar';
+						$node->description = 'foo';
+					}
+
+					return $node;
+				}
+
+			},
+		];
+
+		yield [
+			'/** @param-closure-this Foo $test haha */',
+			'/** @param-closure-this Bar $taste hehe */',
+			new class extends AbstractNodeVisitor {
+
+				public function enterNode(Node $node)
+				{
+					if ($node instanceof ParamClosureThisTagValueNode) {
+						$node->type = new IdentifierTypeNode('Bar');
+						$node->parameterName = '$taste';
+						$node->description = 'hehe';
+					}
+
+					return $node;
+				}
+
+			},
+		];
+
+		yield [
+			'/** @return Foo[abc] */',
+			'/** @return self::FOO[abc] */',
+			new class extends AbstractNodeVisitor {
+
+				public function enterNode(Node $node)
+				{
+					if ($node instanceof ReturnTagValueNode && $node->type instanceof OffsetAccessTypeNode) {
+						$node->type->type = new ConstTypeNode(new ConstFetchNode('self', 'FOO'));
+					}
+
+					return $node;
+				}
+
+			},
+		];
 	}
 
 	/**
@@ -1717,7 +1792,7 @@ class PrinterTest extends TestCase
 	}
 
 	/**
-	 * @return iterable<list{TypeNode, string}>
+	 * @return iterable<array{TypeNode, string}>
 	 */
 	public function dataPrintType(): iterable
 	{
@@ -1804,6 +1879,13 @@ class PrinterTest extends TestCase
 			]),
 			'Foo|Bar|(Baz|Lorem)',
 		];
+		yield [
+			new OffsetAccessTypeNode(
+				new ConstTypeNode(new ConstFetchNode('self', 'TYPES')),
+				new IdentifierTypeNode('int')
+			),
+			'self::TYPES[int]',
+		];
 	}
 
 	/**
@@ -1823,7 +1905,7 @@ class PrinterTest extends TestCase
 	}
 
 	/**
-	 * @return iterable<list{PhpDocNode, string}>
+	 * @return iterable<array{PhpDocNode, string}>
 	 */
 	public function dataPrintPhpDocNode(): iterable
 	{
